import react, { useCallback, useState } from 'react';
import { useDispatch, useSelector } from 'react-redux';

import _ from 'lodash';

import { slippyActions } from '../store/slippy';
import { mouseHandlerActions } from '../store/mouse-handler';

interface MouseHandlerProps {
  children: any;
}

const MouseHandler: react.FC<MouseHandlerProps> = (
  props: MouseHandlerProps
) => {
  const dispatch = useDispatch();

  interface MouseState {
    down: boolean;
    starting: { x: number; y: number };
    timestamp: number;
  }

  const mouseState = useSelector(
    (state: { mouseHandler: MouseState }) => state.mouseHandler
  );

  console.log('MouseHandler, mouseState: ' + JSON.stringify(mouseState));

  const genericHandler = (event: any) => {
    console.log(`Log - Event:  ${event.type}`);
    if (event.pageX !== undefined) {
      console.log(`Mouse : ${event.pageX}, ${event.pageY}`);
      console.log(
        `mouseState: ' ${JSON.stringify(mouseState)} (+${
          Date.now() - mouseState.timestamp
        }ms) `
      );
      return;
    }
  };

  const mouseLeaveHandler = (event: any) => {
    genericHandler(event);
  //  throtteledMouseMoveHandler.cancel();
    dispatch(mouseHandlerActions.init());
  };

  const mouseDownHandler = (event: any) => {
    event.preventDefault();
    genericHandler(event);
    dispatch(
      mouseHandlerActions.set({
        down: true,
        starting: { x: event.pageX, y: event.pageY },
        timestamp: Date.now(),
      })
    );
  };

  const mouseUpHandler = (event: any) => {
    genericHandler(event);
    event.preventDefault();
    dispatch(mouseHandlerActions.init());
  };

  const mouseMoveHandler = (event: any) => {
    event.preventDefault();
    if (mouseState.down && (Date.now() - mouseState.timestamp) > 50 ) {
      genericHandler(event);
      console.log(
        `dispatch ${JSON.stringify({
          x: event.pageX - mouseState.starting.x,
          y: event.pageY - mouseState.starting.y,
        })}`
      );
      dispatch(
        slippyActions.translate({
          x: event.pageX - mouseState.starting.x,
          y: event.pageY - mouseState.starting.y,
        })
      );

      dispatch(
        mouseHandlerActions.set({
          down: true,
          starting: {
            x: event.pageX,
            y: event.pageY,
          },
          timestamp: Date.now(),
        })
      );
    }
  };

  // const throtteledMouseMoveHandler = useCallback(
  //   _.throttle(mouseMoveHandler, 50),
  //   [mouseState.down, mouseState.starting.x, mouseState.starting.y]
  // );

  

  const doubleClickHandler = (event: any) => {
    genericHandler(event);
    dispatch(
      slippyActions.scale({
        factor: 2,
        center: { x: event.pageX, y: event.pageY },
      })
    );
  };

  return (
    <div
      className='viewport mouse-handler'
      onMouseDown={mouseDownHandler}
      onMouseMove={mouseMoveHandler}
      onMouseUp={mouseUpHandler}
      onMouseLeave={mouseLeaveHandler}
      onDoubleClick={doubleClickHandler}
    >
      {props.children}
    </div>
  );
};

export default MouseHandler;
