import react from 'react';
import { useSelector } from 'react-redux';

import MouseHandler from './mouse-handler';
import Layer from './layer';

import '../theme/viewport.css';
import SingleTouchHandler from './single-touch-handler';
import DoubleTouchHandler from './double-touch-handler';

export interface Point {
  x: number;
  y: number;
}

export interface Translation {
  translate: Point;
}

export interface Scale {
  scale: {
    center: Point;
    factor: number;
  };
}


interface ViewportProps {
  children: any;
}

export interface ViewportState {
  scale: number;
  translation: Point;
}

const Viewport: react.FC<ViewportProps> = (props: ViewportProps) => {
  //console.log(`--- Rendering viewport, props: ${JSON.stringify(props)} ---`);

  const state = useSelector(
    (globalState: { slippy: ViewportState }) => globalState.slippy
  );

  return (
    <div className='viewport'>
      <MouseHandler>
        <SingleTouchHandler>
          <DoubleTouchHandler>
            <Layer viewportState={state}>{props.children}</Layer>
          </DoubleTouchHandler>
        </SingleTouchHandler>
      </MouseHandler>
    </div>
  );
};

export default Viewport;
